#include <Keyboard.h>

#define NOTE_C4  262    //These definitions specify the frequency of each note.
#define NOTE_D4  294
#define NOTE_E4  330
#define NOTE_F4  349
#define NOTE_G4  392
#define NOTE_A4  440
#define NOTE_B4  494
#define NOTE_C5  523

#define BUTTON 4
#define NUM_LED 6  
#define MAX 500   //Maximum posible reading. >>>> TWEAK THIS VALUE!! <<<<
#define MIN 100    //Minimum posible reading. >>>> TWEAK THIS VALUE!! <<<<

int channel = 1;   //Determines what channel the musical box is in.

int c4Note;
int c4Duration = 600; //Determines the duration of each note in channel 4.
int c4Scale[]{        //Inclues all the notes you can play on channel 4.
  NOTE_F4, NOTE_G4, NOTE_A4, NOTE_C5
  };
int c2Note;
int c2Duration = 300; //Determines the duration of each note in channel 2.
int c2Scale[]{        //Inclues all the notes you can play on channel 2.
  NOTE_C4, NOTE_D4, NOTE_E4, NOTE_F4, NOTE_G4, NOTE_A4, NOTE_B4
  };
int melody[] = {        //Includes the notes in the melody for "Mary had a Little Lamb".
  NOTE_A4, NOTE_G4, NOTE_F4, NOTE_G4, NOTE_A4, NOTE_A4, NOTE_A4, 
  NOTE_G4, NOTE_G4, NOTE_G4, NOTE_A4, NOTE_C5, NOTE_C5, 
  NOTE_A4, NOTE_G4, NOTE_F4, NOTE_G4, NOTE_A4, NOTE_A4, NOTE_A4,
  NOTE_G4, NOTE_G4, NOTE_A4, NOTE_G4, NOTE_F4
};
int noteDurations[] = { //Determines the duration of each note in the melody.
  4, 4, 4, 4, 4, 4, 2, 4, 4, 2, 4, 4, 2, 4, 4, 4, 4, 4, 4, 2, 4, 4, 4, 4, 2
};
int distortion;   //Used later for distoriting a melody.
int note;

int c5Scale[]{    //Inclues all the notes you can play on channel 4.
  291, 308, 326, 346, 366, 388, 411
};
int c5Note;
int c5Duration = 300;    //Determines the duration of each note in channel 2.

int thisNote = 0;   //Used later for changing between channels.
int buttonState;
int oldButtonState;

int reading[10];    // Used later for reading the electrical signals in your arm. 
int finalReading;
byte litLeds = 0;
byte multiplier = 1;
byte leds[] = {8, 9, 10, 11, 12, 13};

///////////////////////////////////////////////////////////////////////////////////////////////////

void setup() {
  pinMode(BUTTON, INPUT);   //Sets the button as an input.

  Serial.begin(9600); 
  for(int i = 0; i < NUM_LED; i++){     //Sets the LEDs as outputs.
    pinMode(leds[i], OUTPUT);
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////////

void loop() {
  buttonState = digitalRead(BUTTON);
  if ((buttonState == HIGH) && (oldButtonState == LOW)){    //If the button used to not be pressed an is now pressed, change
    if (channel < 6){                                       //the channel and output a tone to indicate that it has changed.
      channel += 1;
      tone(8, 200, 300);
      digitalWrite(leds[channel-1], HIGH);
      delay(300);
    }
    else{
      channel = 1;
      tone(8, 200, 300);
      digitalWrite(leds[0], HIGH);
      delay(300);
    }
  }
  oldButtonState = buttonState;
  
///////////////////////////////////////////////////////////////////////////////////////////////////

for(int i = 0; i < 10; i++){    //Take ten readings in ~0.02 seconds.
    reading[i] = analogRead(A0) * multiplier;
    delay(2);
  }
  for(int i = 0; i < 10; i++){   //Average the ten readings.
    finalReading += reading[i];
  }
  finalReading /= 10;
  for(int j = 0; j < NUM_LED; j++){   //Write all LEDs low.
    digitalWrite(leds[j], LOW);
  }
  Serial.print(finalReading);
  Serial.print("\t");
  finalReading = constrain(finalReading, 0, MAX);
  litLeds = map(finalReading, 0, MAX, 0, NUM_LED);
  Serial.println(litLeds);
  for(int k = 0; k < litLeds; k++){
    digitalWrite(leds[k], HIGH);
  }
    
///////////////////////////////////////////////////////////////////////////////////////////////////

if (channel == 1){
   if(finalReading > MIN){
       tone(8, finalReading/1.5, 100); //Output a tone equal to the final reading divided by 1.5.
       delay(100);
   }
  }
  
///////////////////////////////////////////////////////////////////////////////////////////////////

  if (channel == 2){
    if (finalReading > MIN) {
      c2Note = map(finalReading, MIN, MAX, 0, 6);   //Map every frequency to a number between 0 and 6.
      tone(8, c2Scale[c2Note], c2Duration);         //Convert that number to a note and play it.
      delay(c2Duration);
    }
  }

///////////////////////////////////////////////////////////////////////////////////////////////////

  if (channel == 3){
    if (finalReading > MIN) {
      c2Note = map(finalReading, MIN, MAX, 0, 6);   //Map every frequency to a number between 0 and 6.
      tone(8, c2Scale[c2Note]*0.4 +8, c2Duration);         //Convert that number to a note and play it.
      delay(c2Duration);
    }    
  }

///////////////////////////////////////////////////////////////////////////////////////////////////
  
  if (channel == 4){
    if (finalReading > MIN) {
      c5Note = map(finalReading, MIN, MAX, 0, 6);   //Map every frequency to a number between 0 and 6.
      tone(8, c5Scale[c5Note], c5Duration);         //Convert that number to a note and play it.
      delay(c5Duration);
    }  
  }

  
///////////////////////////////////////////////////////////////////////////////////////////////////

  if (channel == 5){      
    //To calculate the note duration, take one second divided by the note type.
    int noteDuration = (1400 - finalReading*2.5) / noteDurations[thisNote];
    tone(8, melody[thisNote], noteDuration);
    delay(noteDuration * 1);   //To distinguish the notes, set a minimum time between them.
    noTone(8);
    if (thisNote == 24){    //If the melody has ended, go back to the first note.
      delay(400);
      thisNote = -1;
      }
    thisNote ++;
    }
    
///////////////////////////////////////////////////////////////////////////////////////////////////

  if (channel == 6){      
    int distortion = map(finalReading, MIN, MAX, 7, 14);
    Serial.println(distortion);
    //To calculate the note duration, take one second divided by the note type.
    int noteDuration = 1000 / noteDurations[thisNote];
    tone(8, (melody[thisNote]*distortion)/10, noteDuration);
    delay(noteDuration * 1);   //To distinguish the notes, set a minimum time between them.
    noTone(8);
    if (thisNote == 24){    //If the melody has ended, go back to the first note.
      delay(400);
      thisNote = -1;
      }
    thisNote ++;
    }
}
